<?php

// $Id: media_mover_ftp_harvest.inc,v 1.1.2.13 2009/02/17 05:55:07 arthuregg Exp $;

/**
 * Run the FTP harvest.
 * @param array $configuration
 *   current media mover configuration
 * @return array of files
 *
 * */
function media_mover_ftp_harvest($conf) {
  // build the ftp connection
  $ftp_conn = ftp_connect($conf['mm_ftp_host']);
  $ftp_login = ftp_login($ftp_conn, $conf['mm_ftp_user'], $conf['mm_ftp_pass']);
  $ftp_list = ftp_rawlist($ftp_conn, $conf['mm_ftp_dir']);

  // check the connection to the server
  if (! $ftp_conn) {
    watchdog(MM_FTP_WATCHDOG_TYPE, t('Unable to connect to server %server', array('%server' => $conf['mm_ftp_host'])), WATCHDOG_ERROR);
    return;
  }
  elseif (! $ftp_login) {
    watchdog(MM_FTP_WATCHDOG_TYPE, t('Unable to login to server %server with username %user', array('%server' => $conf['mm_ftp_host'], '%user' => $conf['mm_ftp_user'])), WATCHDOG_ERROR);
    return;
  }
  elseif ($ftp_list === FALSE) {
    watchdog(MM_FTP_WATCHDOG_TYPE, t('Unable to get file list from server %server with username %user', array('%server' => $conf['mm_ftp_host'], '%user' => $conf['mm_ftp_user'])), WATCHDOG_ERROR);
    return;
  }

  // make sure we have files being handed back
  $count = count($ftp_list);
  if ($count == 0) {
    return;
  }

  $del = $conf['mm_ftp_delete_files'];
  $file_types = explode(' ' , $conf['mm_ftp_allowed_files']);

  $date = date('Y-m-d H:i:s');
  $logfile = '';
  foreach($ftp_list as $file_item) {
    $file_data = mm_ftp_parse_file($file_item, $conf);
    // get each file and add it on to the items
    if ($item = media_mover_ftp_harvest_ftp_file($ftp_conn, $file_data, $conf)) {
      $items[] = $item;
    }
  }
  ftp_close($ftp_conn);
  return $items;
}


/**
 * This goes through each of the files from the ftp server and harvests individual files
 * @param object $ftp_conn this is the connection to the ftp server
 * @param array $info this needs to be renamed to something more clear
 * @param array $configuration is the configuration currently running
 * @return array should be just a single item
 */
function media_mover_ftp_harvest_ftp_file(&$ftp_conn, $info, $configuration) {
  // Get each item and save it locally
  if ($item = media_mover_ftp_create_file($ftp_conn, $info, $configuration)) {
    // should we delete the file after we've harvested it?
    if ($configuration['mm_ftp_delete_files']) {
      if (! ftp_delete($ftp_conn, $info['dir'] ."/". $info['name'])) {
        watchdog(MM_FTP_WATCHDOG_TYPE, t('Error deleting file: @file', array('@file' => $info['name'])), WATCHDOG_ERROR);
        return FALSE;
      }
    }
    // return the item if we have it
    if ($item) {
      return $item;
    }
  }
  return FALSE;
}


/**
 * function for saving the file from the FTP server to the local drupal directory
 * @param object $ftp_conn is the ftp connection
 * @param array $info is the info array
 * @param array $configuration is the media mover configuration
 * @return array $item is a item array
 */
function media_mover_ftp_create_file(&$ftp_conn, $info, $configuration) {

  // does this file have a token, or are we allowed ot harvest files without tokens?
  if (! media_mover_ftp_token_extract($info['name'], 'token') && ! $configuration['mm_ftp_harvest_without_tokens'] ) { 
    // should we delete files that do not have a token from the ftp server?
    if ($configuration['mm_ftp_delete_files_no_token']) {
      ftp_delete($ftp_conn, $info['dir'] ."/". $info['name']);
      watchdog(MM_FTP_WATCHDOG_TYPE, t('Deleted FTP file without token.'), WATCHDOG_NOTICE);
    }
    // now we need to return because we are not supposed to harvest this file type
    return false;    
  }

  // get the file name from the harvested file
  $local_file = media_mover_ftp_token_extract($info['name'], 'file');
  
  // get file path from media mover
  $file_location = media_mover_api_dir_path() . MM_FTP_FILES_SUBDIRECTORY;

  // look out for dupes
  $local_file_path = file_create_filename($local_file, $file_location);

  // only harvested file types that match the requested types
  if (! media_mover_api_file_validate_extensions($local_file, $configuration['mm_ftp_allowed_files'])) {
    watchdog(MM_FTP_WATCHDOG_TYPE, 'FTP file harvest failed: unsupported file extension !file', array('!file' => $local_file_path));
    return false;
  }

  // download the file via FTP
  if (! ftp_get($ftp_conn, $local_file_path, $info['dir'] ."/". $info['name'], FTP_BINARY)) {
    watchdog(MM_FTP_WATCHDOG_TYPE, t('FTP file harvest failed to download file: @file', array('@file' => $local_file_path)));
    return false;
  }

  // @TODO we may want to lock this file down more, not sure. This is a potential
  //       security risk because we don't really know what the file is that has been
  //       downloaded. We can use the mimedetect library to help us determine what kind
  //       of file it is and remove it if we think it's dangerous.

  // types we support
  $types = array('video', 'image', 'audio', 'application/octet-stream');
  // get the mime value of the file we downloaded
  $mime = file_get_mimetype($local_file);
  foreach($types as $type) {
    if (stristr($mime, $type)) {
      // we think this is a safe file
      $legal = true;
    }
  }
  if (! $legal) {
    // log the error
    watchdog(MM_FTP_WATCHDOG_TYPE, t('Harvested an illegal MIME type: @mime for @file', array('@mime' => $mime, '@file' => $local_file_path)));
    // delete the LOCAL file
    file_delete($local_file);
    return;
  }

  // log the harvested file
  watchdog(MM_FTP_WATCHDOG_TYPE, 'FTP file harvested: '. $local_file_path);

  $item['filesize'] = $info['size'];
  $item['filename'] = $local_file;
  $item['filepath'] =  $local_file_path;
  $item['harvest_file'] = $local_file_path;
  $item['node']->title = $local_file;

  // get a user id from the token
  if ($user = media_mover_ftp_token_check(media_mover_ftp_token_extract($info['name'], 'token'))) {
    $item['user'] = $user;
    $item['node']->uid = $user->uid;
  }

  return $item;
}
