<?php

// $Id: mm_ftp_storage.inc,v 1.1.2.1 2009/01/21 17:21:53 arthuregg Exp $

/**
 * @file
 * This provides storage integration with FTP servers
 */


/**
 * Does the storage operation of the current file
 * @param $file
 * @param $configuration
 * @return unknown_type
 */
function mm_ftp_storage(&$file, $configuration) {
  // get the current file path
  $file_local_path = media_mover_api_config_current_file($file);

  // open the FTP connection
  $ftp_connection = ftp_connect($configuration['mm_ftp_host']);
  $ftp_login = ftp_login($ftp_connection, $configuration['mm_ftp_user'], $configuration['mm_ftp_pass']);
  $ftp_file_list = ftp_rawlist($ftp_connection, $configuration['mm_ftp_dir']);

  // check the connection to the server
  if (! $ftp_connection) {
    watchdog(MM_FTP_WATCHDOG_TYPE, t('Unable to connect to server %server', array('%server' => $configuration['mm_ftp_host'])), WATCHDOG_ERROR);
    return;
  }
  elseif (! $ftp_login) {
    watchdog(MM_FTP_WATCHDOG_TYPE, t('Unable to login to server %server with username %user', array('%server' => $configuration['mm_ftp_host'], '%user' => $configuration['mm_ftp_user'])), WATCHDOG_ERROR);
    return;
  }
  elseif ($ftp_list === FALSE) {
    watchdog(MM_FTP_WATCHDOG_TYPE, t('Unable to get file list from server %server with username %user', array('%server' => $configuration['mm_ftp_host'], '%user' => $configuration['mm_ftp_user'])), WATCHDOG_ERROR);
    return;
  }

  // open the file that is being moved
  $put_file = fopen($file_local_path, 'r');

  // build the FTP file path
  $file_ftp_path = $configuration['mm_ftp_dir'] .'/'. basename($file_local_path);

  // get a unique file name for this file
  $file_name = mm_ftp_file_check($file_ftp_path, $ftp_file_list, $configuration);

  // put this file on the ftp server
  if (! ftp_fput($ftp_connection, $file_name, $put_file, FTP_BINARY)) {
    watchdog(MM_FTP_WATCHDOG_TYPE, 'Unable to copy file to FTP server', WATCHDOG_ERROR);
    return false;
  }

  // close the connections
  fclose($put_file);
  ftp_close($ftp_connection);

  // what kind of file path are we returning?
  if ($configuration['mm_ftp_url'] == '<none>') {
    $file_path = 'ftp://'. $file_ftp_path;
  }
  else {
    $file_path = $configuration['mm_ftp_url'] . $file_ftp_path;
  }

  // @TODO
  // do we want to delete the files here?

  return $file_path;
}



/* ************************************************ */
/* INTERNAL FUNCTIONS                               */
/* ************************************************ */

function mm_ftp_file_check($file_path, $ftp_file_list, $configuration) {
  static $files, $i;

  // build a list of files
  if (! $files) {
    $files = array();
    // sort through all of the files found and look for the current file path
    foreach ($ftp_file_list as $file_item) {
      $file_item = mm_ftp_parse_file($file_item, $configuration);
      $files[$file_item['name']] = $file_item;
    }
    $i = 0;
  }
//print_r($file_path);
 // print_r($files);

  // check to see if we have this file name
  if ($files[basename($file_path)] ) {
    $info = pathinfo($file_path);
    $file_path = $info['dirname'] .'/'. $info['basename'] .'_'. $i .'.'. $info['extension'];
    // recurse till we have a unique file
    $file_path = mm_ftp_file_check($file_path, $ftp_file_list, $configuration);
  }

  return $file_path;
}